// 
// SVEHaTypCross is Copyright (C) 2013, Sylvain Vervoort <stocata.org>.
// stocata.org reserves the right to modify/overwrite this NinjaScript with each release.
// Release V1.0 January, 2013.

#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
/// <summary>
/// A typical and heikin-ashi price crossover color coding the candle chart.
/// </summary>
[Description("")]
public class SVEHaTypCross : Indicator
{
    #region Variables
	
	private int haCaverage 		=	8; 	// Default heikin-ashi Closing price average
    private int typicalaverage 	=	5; 	// Default typical price average

	private DataSeries 		haOpen;		// heikin ashi open price
	private DataSeries 		haC;		// heikin ashi closing price
	private DataSeries 		typical;	// typical price
	private DataSeries 		AVGtyp;		// typical price average
	private DataSeries 		AVGhaC;		// heikin-ashi average
	private DataSeries 		cross;		// typical average crossing HA average
	
	private Color candlefillup		= 	Color.White;
	private Color candlednfill		= 	Color.Gray;
	private Color candleupfill		= 	Color.LimeGreen;
	private Color outlineup			= 	Color.LimeGreen;
	private Color outlinedn			= 	Color.Black;

    #endregion

    /// <summary>
    /// Configuring the indicator called once before any bar data.
    /// </summary>
    protected override void Initialize()
    {	
		
		haOpen 	= new DataSeries(this);
		haC 	= new DataSeries(this);
		typical = new DataSeries(this);
		AVGtyp	= new DataSeries(this);
		AVGhaC	= new DataSeries(this);
		cross	= new DataSeries(this);
		
		PaintPriceMarkers 	= false;
		Overlay				= true;

    }

    /// <summary>
    /// Called on each bar update event (incoming tick)
    /// </summary>
    protected override void OnBarUpdate()
    {	
		if (CurrentBar < 1) // minimum 2 bars required
			return;
		
		// Create crossover data sets
		haOpen.Set((((Open[1] + High[1] + Low[1] + Close[1]) / 4) + haOpen[1]) / 2);
		haC.Set(((Open[0] + High[0] + Low[0] + Close[0]) / 4 + haOpen[0] + 
				  Math.Max(High[0], haOpen[0]) + Math.Min(Low[0], haOpen[0])) / 4);
		typical.Set((High[0] + Low[0] + Close[0]) / 3);

		// Create the averages
		AVGtyp.Set(EMA(typical, typicalaverage)[0]);
		AVGhaC.Set(EMA(haC, haCaverage)[0]);

		// Crossover condition
		if(AVGtyp[0] > AVGhaC[0] && Close[0] > Open[0]) cross[0] = 1;
		else if(AVGtyp[0] < AVGhaC[0] && Close[0] < Open[0]) cross[0] = 0;
		else cross[0] = cross[1];
		
		// Color coding the candle chart
		if(cross[0] == 1)
		{
			BarColorSeries[0] = candleupfill;
			CandleOutlineColorSeries[0] = outlineup;
			if(ChartControl.ChartStyleType == ChartStyleType.CandleStick)  
				if(Close[0] >= Open[0]) 
			   	   BarColorSeries[0] = candlefillup;
			else BarColorSeries[0] = candleupfill;
		}
		else if(cross[0] == 0)
		{
			BarColorSeries[0] = candlednfill;
			CandleOutlineColorSeries[0] = outlinedn;
			if(ChartControl.ChartStyleType == ChartStyleType.CandleStick)  
				if(Close[0] >= Open[0])  
			   	   BarColorSeries[0] = candlefillup;
			else BarColorSeries[0] = candlednfill;
		}
    }

    #region Properties
	
	/// <summary>
	/// </summary>
	[Description("Heikin-ashi Average DEF = 8")]
	[Category("Parameters")]
	[Gui.Design.DisplayName("1. Heikin-Ashi average")]
	public int AVG__slow
	{
		get { return haCaverage; }
		set { haCaverage = Math.Max(1, value); }
	}

	/// <summary>
	/// </summary>
	[Description("Typical average DEF = 5")]
	[Category("Parameters")]
	[Gui.Design.DisplayName("2. Typical Price Average")]
	public int AVG_fast
	{
		get { return typicalaverage; }
		set { typicalaverage = Math.Max(1, value); }
	}
	
	[Description("Color for an UP candle DEFAULT = White")]
	[Category("Parameters")]
	[Gui.Design.DisplayName("3. Up candle color")]
	public Color Candlefillup
	{
		get { return candlefillup; }
		set { candlefillup = value; }
	}
	[Browsable(false)]
	public string CandlefillupSerialize
	{
 		get { return NinjaTrader.Gui.Design.SerializableColor.ToString(candlefillup); }
 		set { candlefillup = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
	}
	
	[Description("Down candle fill color DEFAULT = Gray")]
	[Category("Parameters")]
	[Gui.Design.DisplayName("4. Down fill color")]
	public Color Candlednfill
	{
		get { return candlednfill; }
		set { candlednfill = value; }
	}
	[Browsable(false)]
	public string CandlednfillSerialize
	{
 		get { return NinjaTrader.Gui.Design.SerializableColor.ToString(candlednfill); }
 		set { candlednfill = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
	}
	
	[Description("Up candle fill color DEFAULT = LimeGreen")]
	[Category("Parameters")]
	[Gui.Design.DisplayName("5. Up fill color")]
	public Color Candleupfill
	{
		get { return candleupfill; }
		set { candleupfill = value; }
	}
	[Browsable(false)]
	public string CandleupfillSerialize
	{
 		get { return NinjaTrader.Gui.Design.SerializableColor.ToString(candleupfill); }
 		set { candleupfill = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
	}	
	
	[Description("Outline Up color DEFAULT = LimeGreen")]
	[Category("Parameters")]
	[Gui.Design.DisplayName("6. Outline Up color")]
	public Color Outlineup
	{
		get { return outlineup; }
		set { outlineup = value; }
	}
	[Browsable(false)]
	public string OutlineupSerialize
	{
 		get { return NinjaTrader.Gui.Design.SerializableColor.ToString(outlineup); }
 		set { outlineup = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
	}	
	
	[Description("Outline Down color DEFAULT = Black")]
	[Category("Parameters")]
	[Gui.Design.DisplayName("7. Outline Down color")]
	public Color Outlinedn
	{
		get { return outlinedn; }
		set { outlinedn = value; }
	}
	[Browsable(false)]
	public string OutlinednSerialize
	{
 		get { return NinjaTrader.Gui.Design.SerializableColor.ToString(outlinedn); }
 		set { outlinedn = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
	}	

	#endregion
}
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private SVEHaTypCross[] cacheSVEHaTypCross = null;

        private static SVEHaTypCross checkSVEHaTypCross = new SVEHaTypCross();

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public SVEHaTypCross SVEHaTypCross(int aVG__slow, int aVG_fast, Color candlednfill, Color candlefillup, Color candleupfill, Color outlinedn, Color outlineup)
        {
            return SVEHaTypCross(Input, aVG__slow, aVG_fast, candlednfill, candlefillup, candleupfill, outlinedn, outlineup);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public SVEHaTypCross SVEHaTypCross(Data.IDataSeries input, int aVG__slow, int aVG_fast, Color candlednfill, Color candlefillup, Color candleupfill, Color outlinedn, Color outlineup)
        {
            if (cacheSVEHaTypCross != null)
                for (int idx = 0; idx < cacheSVEHaTypCross.Length; idx++)
                    if (cacheSVEHaTypCross[idx].AVG__slow == aVG__slow && cacheSVEHaTypCross[idx].AVG_fast == aVG_fast && cacheSVEHaTypCross[idx].Candlednfill == candlednfill && cacheSVEHaTypCross[idx].Candlefillup == candlefillup && cacheSVEHaTypCross[idx].Candleupfill == candleupfill && cacheSVEHaTypCross[idx].Outlinedn == outlinedn && cacheSVEHaTypCross[idx].Outlineup == outlineup && cacheSVEHaTypCross[idx].EqualsInput(input))
                        return cacheSVEHaTypCross[idx];

            lock (checkSVEHaTypCross)
            {
                checkSVEHaTypCross.AVG__slow = aVG__slow;
                aVG__slow = checkSVEHaTypCross.AVG__slow;
                checkSVEHaTypCross.AVG_fast = aVG_fast;
                aVG_fast = checkSVEHaTypCross.AVG_fast;
                checkSVEHaTypCross.Candlednfill = candlednfill;
                candlednfill = checkSVEHaTypCross.Candlednfill;
                checkSVEHaTypCross.Candlefillup = candlefillup;
                candlefillup = checkSVEHaTypCross.Candlefillup;
                checkSVEHaTypCross.Candleupfill = candleupfill;
                candleupfill = checkSVEHaTypCross.Candleupfill;
                checkSVEHaTypCross.Outlinedn = outlinedn;
                outlinedn = checkSVEHaTypCross.Outlinedn;
                checkSVEHaTypCross.Outlineup = outlineup;
                outlineup = checkSVEHaTypCross.Outlineup;

                if (cacheSVEHaTypCross != null)
                    for (int idx = 0; idx < cacheSVEHaTypCross.Length; idx++)
                        if (cacheSVEHaTypCross[idx].AVG__slow == aVG__slow && cacheSVEHaTypCross[idx].AVG_fast == aVG_fast && cacheSVEHaTypCross[idx].Candlednfill == candlednfill && cacheSVEHaTypCross[idx].Candlefillup == candlefillup && cacheSVEHaTypCross[idx].Candleupfill == candleupfill && cacheSVEHaTypCross[idx].Outlinedn == outlinedn && cacheSVEHaTypCross[idx].Outlineup == outlineup && cacheSVEHaTypCross[idx].EqualsInput(input))
                            return cacheSVEHaTypCross[idx];

                SVEHaTypCross indicator = new SVEHaTypCross();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.AVG__slow = aVG__slow;
                indicator.AVG_fast = aVG_fast;
                indicator.Candlednfill = candlednfill;
                indicator.Candlefillup = candlefillup;
                indicator.Candleupfill = candleupfill;
                indicator.Outlinedn = outlinedn;
                indicator.Outlineup = outlineup;
                Indicators.Add(indicator);
                indicator.SetUp();

                SVEHaTypCross[] tmp = new SVEHaTypCross[cacheSVEHaTypCross == null ? 1 : cacheSVEHaTypCross.Length + 1];
                if (cacheSVEHaTypCross != null)
                    cacheSVEHaTypCross.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheSVEHaTypCross = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.SVEHaTypCross SVEHaTypCross(int aVG__slow, int aVG_fast, Color candlednfill, Color candlefillup, Color candleupfill, Color outlinedn, Color outlineup)
        {
            return _indicator.SVEHaTypCross(Input, aVG__slow, aVG_fast, candlednfill, candlefillup, candleupfill, outlinedn, outlineup);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public Indicator.SVEHaTypCross SVEHaTypCross(Data.IDataSeries input, int aVG__slow, int aVG_fast, Color candlednfill, Color candlefillup, Color candleupfill, Color outlinedn, Color outlineup)
        {
            return _indicator.SVEHaTypCross(input, aVG__slow, aVG_fast, candlednfill, candlefillup, candleupfill, outlinedn, outlineup);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.SVEHaTypCross SVEHaTypCross(int aVG__slow, int aVG_fast, Color candlednfill, Color candlefillup, Color candleupfill, Color outlinedn, Color outlineup)
        {
            return _indicator.SVEHaTypCross(Input, aVG__slow, aVG_fast, candlednfill, candlefillup, candleupfill, outlinedn, outlineup);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public Indicator.SVEHaTypCross SVEHaTypCross(Data.IDataSeries input, int aVG__slow, int aVG_fast, Color candlednfill, Color candlefillup, Color candleupfill, Color outlinedn, Color outlineup)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.SVEHaTypCross(input, aVG__slow, aVG_fast, candlednfill, candlefillup, candleupfill, outlinedn, outlineup);
        }
    }
}
#endregion
