// 
// SVEHaTypCrossInd is Copyright (C) 2013, Sylvain Vervoort <stocata.org>.
// stocata.org reserves the right to modify/overwrite this NinjaScript with each release.
// Release V1.1 January, 2013.

#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
/// <summary>
/// A typical and heikin-ashi price crossover digital indicator.
/// </summary>
[Description("Typical price and heikin ashi crossover digital indicator")]
public class SVEHaTypCrossInd : Indicator
{
    #region Variables
	
	private int haCaverage 		=	8; 	// Default heikin-ashi Closing price average
    private int typicalaverage 	=	5; 	// Default typical price average

	private DataSeries 		haOpen;		// heikin ashi open price
	private DataSeries 		haC;		// heikin ashi closing price
	private DataSeries 		typical;	// typical price
	private DataSeries 		AVGtyp;		// typical price average
	private DataSeries 		AVGhaC;		// heikin-ashi average
	private	DataSeries 		cross;		// typical average crossing HA average
	
    #endregion

    /// <summary>
    /// Configuring the indicator called once before any bar data.
    /// </summary>
    protected override void Initialize()
    {	
		Add(new Plot(Color.DodgerBlue, "Cross Ind"));
		
		haOpen 	= new DataSeries(this);
		haC 	= new DataSeries(this);
		typical = new DataSeries(this);
		AVGtyp	= new DataSeries(this);
		AVGhaC	= new DataSeries(this);
		cross	= new DataSeries(this);
		
		PaintPriceMarkers 	= false;
		Overlay				= false;

    }

    /// <summary>
    /// Called on each bar update event (incoming tick)
    /// </summary>
    protected override void OnBarUpdate()
    {	
		if (CurrentBar < 1) // minimum 2 bars required
			return;
		
		// Create crossover data sets
		haOpen.Set((((Open[1] + High[1] + Low[1] + Close[1]) / 4) + haOpen[1]) / 2);
		haC.Set(((Open[0] + High[0] + Low[0] + Close[0]) / 4 + haOpen[0] + 
				  Math.Max(High[0], haOpen[0]) + Math.Min(Low[0], haOpen[0])) / 4);
		typical.Set((High[0] + Low[0] + Close[0]) / 3);

		// Create the averages
		AVGtyp.Set(EMA(typical, typicalaverage)[0]);
		AVGhaC.Set(EMA(haC, haCaverage)[0]);

		// Crossover condition
		if(AVGtyp[0] > AVGhaC[0] && Close[0] > Open[0]) cross[0] = 1;
		else if(AVGtyp[0] < AVGhaC[0] && Close[0] < Open[0]) cross[0] = 0;
		else cross[0] = cross[1];
		Value.Set(cross[0]);
    }

    #region Properties
	
	/// <summary>
	/// </summary>
	[Description("Heikin-ashi Average DEF = 8")]
	[GridCategory("Parameters")]
	[Gui.Design.DisplayName("1. Heikin-Ashi average")]
	public int AVG__slow
	{
		get { return haCaverage; }
		set { haCaverage = Math.Max(1, value); }
	}

	/// <summary>
	/// </summary>
	[Description("Typical average DEF = 5")]
	[GridCategory("Parameters")]
	[Gui.Design.DisplayName("2. Typical Price Average")]
	public int AVG_fast
	{
		get { return typicalaverage; }
		set { typicalaverage = Math.Max(1, value); }
	}
		
	#endregion
}
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private SVEHaTypCrossInd[] cacheSVEHaTypCrossInd = null;

        private static SVEHaTypCrossInd checkSVEHaTypCrossInd = new SVEHaTypCrossInd();

        /// <summary>
        /// Typical price and heikin ashi crossover digital indicator
        /// </summary>
        /// <returns></returns>
        public SVEHaTypCrossInd SVEHaTypCrossInd(int aVG__slow, int aVG_fast)
        {
            return SVEHaTypCrossInd(Input, aVG__slow, aVG_fast);
        }

        /// <summary>
        /// Typical price and heikin ashi crossover digital indicator
        /// </summary>
        /// <returns></returns>
        public SVEHaTypCrossInd SVEHaTypCrossInd(Data.IDataSeries input, int aVG__slow, int aVG_fast)
        {
            if (cacheSVEHaTypCrossInd != null)
                for (int idx = 0; idx < cacheSVEHaTypCrossInd.Length; idx++)
                    if (cacheSVEHaTypCrossInd[idx].AVG__slow == aVG__slow && cacheSVEHaTypCrossInd[idx].AVG_fast == aVG_fast && cacheSVEHaTypCrossInd[idx].EqualsInput(input))
                        return cacheSVEHaTypCrossInd[idx];

            lock (checkSVEHaTypCrossInd)
            {
                checkSVEHaTypCrossInd.AVG__slow = aVG__slow;
                aVG__slow = checkSVEHaTypCrossInd.AVG__slow;
                checkSVEHaTypCrossInd.AVG_fast = aVG_fast;
                aVG_fast = checkSVEHaTypCrossInd.AVG_fast;

                if (cacheSVEHaTypCrossInd != null)
                    for (int idx = 0; idx < cacheSVEHaTypCrossInd.Length; idx++)
                        if (cacheSVEHaTypCrossInd[idx].AVG__slow == aVG__slow && cacheSVEHaTypCrossInd[idx].AVG_fast == aVG_fast && cacheSVEHaTypCrossInd[idx].EqualsInput(input))
                            return cacheSVEHaTypCrossInd[idx];

                SVEHaTypCrossInd indicator = new SVEHaTypCrossInd();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.AVG__slow = aVG__slow;
                indicator.AVG_fast = aVG_fast;
                Indicators.Add(indicator);
                indicator.SetUp();

                SVEHaTypCrossInd[] tmp = new SVEHaTypCrossInd[cacheSVEHaTypCrossInd == null ? 1 : cacheSVEHaTypCrossInd.Length + 1];
                if (cacheSVEHaTypCrossInd != null)
                    cacheSVEHaTypCrossInd.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheSVEHaTypCrossInd = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Typical price and heikin ashi crossover digital indicator
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.SVEHaTypCrossInd SVEHaTypCrossInd(int aVG__slow, int aVG_fast)
        {
            return _indicator.SVEHaTypCrossInd(Input, aVG__slow, aVG_fast);
        }

        /// <summary>
        /// Typical price and heikin ashi crossover digital indicator
        /// </summary>
        /// <returns></returns>
        public Indicator.SVEHaTypCrossInd SVEHaTypCrossInd(Data.IDataSeries input, int aVG__slow, int aVG_fast)
        {
            return _indicator.SVEHaTypCrossInd(input, aVG__slow, aVG_fast);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Typical price and heikin ashi crossover digital indicator
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.SVEHaTypCrossInd SVEHaTypCrossInd(int aVG__slow, int aVG_fast)
        {
            return _indicator.SVEHaTypCrossInd(Input, aVG__slow, aVG_fast);
        }

        /// <summary>
        /// Typical price and heikin ashi crossover digital indicator
        /// </summary>
        /// <returns></returns>
        public Indicator.SVEHaTypCrossInd SVEHaTypCrossInd(Data.IDataSeries input, int aVG__slow, int aVG_fast)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.SVEHaTypCrossInd(input, aVG__slow, aVG_fast);
        }
    }
}
#endregion
